// Program to capture and train new users to system
// Andrew Palmer (ajp294)
// 31 October 2016

#include <fstream>
#include <iostream>
#include <sys/types.h>
#include <sys/stat.h>
#include <sstream>



std::string newUser(void)
{
   // Ask for new user to register to system
   std::string user_name;
   std::cout << "Please enter username, without spaces, for registry to user database: " << std::endl;
   std::cin >> user_name;

   std::cout << "You entered: " << user_name << std::endl;

   // Add new user to users file
   std::ofstream outfile;
   outfile.open("Users.txt", std::ios::app);
   outfile << user_name << std::endl;

   // create new user's directory for positive training images
   std::ostringstream directory_name;
   directory_name << "../training/" << user_name;
   std::cout << directory_name.str() << std::endl;
   mkdir(directory_name.str().c_str(), S_IRWXU | S_IRWXG | S_IROTH | S_IXOTH);

   // Capture images of new user
   int number_of_images;
   bool success;
   cv::Mat image;
   cv::Rect roi;
   std::cout << "How many training images? " << std::endl;
   std::cin >> number_of_images;
   for (int i = 0; i < number_of_images; i++ ) {
      success = false;
      while (!success) {
         // capture image from picamera
         image = captureImage();

         // Detect face, then crop and resize accordingly
         roi = detectFace(image);
         if (roi.width != 0) success = true;
      }
      cv::Mat processedImage = cropAndResizeImage(image, roi);

      // create filenames and save processed images to training directory
      std::ostringstream filename;
      filename << "../training/" << directory_name.str() << "/" <<  user_name << "_" << i << ".jpg";
      std::cout << "wrote files: " << filename.str() << std::endl;
      cv::imwrite(filename.str(), processedImage);
   }

   return user_name;
}

Ptr<FaceRecognizer> trainModel(std::string user_name) // std::string fn_csv, std::string user_name) 
{  
   vector<Mat> images;
   vector<int> labels;
   std::string fn_csv;
   clock_t start;
   double duration;

   // fn_csv = generateCsv();
   // generate CSV file
   system("python ../training/create_csv.py ../training/");
   fn_csv = "training.csv";
   
   // Read in the data from CSV file, which will have all users and negatives
   try {
      read_csv(fn_csv, images, labels);
   }
   catch (cv::Exception& e) {
      std::cerr << "Error opening file\"" << fn_csv << "\". Reason: " << e.msg << std::endl;
      exit(1);
   }
   
   // Need enough images for training
   if (images.size() <= 1) {
      std::cerr << "Need more than 1 images to train model" << std::endl;
   }
   
   // train model
   std::cout << "Training EigenFaceRecognizer Model" << std::endl;
   Ptr<FaceRecognizer> model = createEigenFaceRecognizer();
   // time training
   start = clock();
   model->train(images, labels);
   duration = (clock() - start) / (double) CLOCKS_PER_SEC;
   std::cout << "Training model duration: " << duration << " seconds" << std::endl;
   
   // save model results
   model->save("training_" + user_name + ".xml");
   
   // return model object
   return model;
}

std::string generateCsvAndTrain(std::string user_name)
{
   // call python script to generate csv file
   std::string python_command = "python ../training/create_csv.py ../training/";
   system(python_command.c_str());
   // trainModel("training.csv", user_name);

   return "training_" + user_name + ".xml";
}

void captureAndTrain(void) 
{
   std::string new_user;

   new_user = newUser();
   generateCsvAndTrain(new_user);
}
