/**
 * RPi program to send "buzz in" message to arduino.
 * This program uses the RF24 library available on GitHub.
 */


#include <cstdlib>
#include <iostream>
#include <sstream>
#include <string>
#include <unistd.h>
#include <RF24/RF24.h>

using namespace std;

#define RETRIES 5

// Hardware configuration

// Palmer's RPi setup
// FINDME: do defines use BCM or board numbering?
//    below are board numbering macros
RF24 radio(RPI_V2_GPIO_P1_22, RPI_V2_GPIO_P1_24, BCM2835_SPI_SPEED_8MHZ);


/********************************/

// Radio pipe addresses for the 2 nodes to communicate.
const uint8_t pipes[][6] = {"1Node","2Node"};

unsigned long received_message;


// Prepare Radio module
void setup(void) 
{
   // printf("Preparing interface\n");
   radio.begin();
   radio.setRetries(15, 15);

   // FINDME: removing print to make web page cleaner
   // radio.printDetails();
   radio.openWritingPipe(pipes[0]);
   radio.openReadingPipe(1, pipes[1]);
}

// send a message to arduino
bool sendMessage(int action)
{
   radio.stopListening();
   unsigned long message = action;
   printf("Now sending request type: %lu...\n", message);

   // Send the message
   bool status = radio.write(&message, sizeof(unsigned long));
   // check message send status
   if (!status) {
      printf("failed to send.. \n\r");
   }
   else {
      printf("Sent!\n");
   }

   // Listen for ACK
   radio.startListening();

   // give listen a timeout
   unsigned long wait_start_time = millis();
   bool timeout = false;
   
   while(!radio.available() && !timeout) {
      if (millis() - wait_start_time > 1000) {
         timeout = true;
      }
   }

   if (timeout) {
      printf("timeout on ACK reached\n");
      return false;
   }
   else {
      // received ack in time
      radio.read(&received_message, sizeof(unsigned long));
      if (received_message == 2) {
         printf("ACK from arduino: success! Response code: %ld\n", received_message);
      }
      return true;
   }

}

int main(int argc, char** argv)
{
   // headers for CGI interface
   printf("Content-Type: text/plain;charset=us-ascii\n\n");
   printf("\n<h1>Buzzing in!</h>\n");
   printf("yay!\n");

   // value for when Tx to Arduino is successful
   bool isTxSuccess = false;
   // unsigned long message = 17;
 
   // set up radio
   setup();

   for (int i = 0; i < RETRIES; i++) {
      if (!isTxSuccess) {
         // FINDME: What exactly are we going to send?
         isTxSuccess = sendMessage(1);
         printf("success on attempt #%d\n", i + 1);
      }
   }

   if (isTxSuccess) {
      return 0;
   }
   else {
      return 2;
   }

} // end main

