// Program to detect a face using haar cascade classifier
// Andrew Palmer (ajp294)
// 21 October 2016
// Used tutorial on opencv for eigenfacerecognizer 


using namespace cv;

static void read_csv(const string& filename, vector<Mat>& images, vector<int>& labels, char separator = ';')
{
   std::ifstream file(filename.c_str(), ifstream::in);

   if (!file) {
      std::cerr << "error loading file" << std::endl;
   }

   std::string line, path, classLabel;
   while (getline(file, line)) {
      stringstream liness(line);
      getline(liness, path, separator);
      getline(liness, classLabel);
      if (!path.empty() && !classLabel.empty()) {
         images.push_back(imread(path, 0));
         labels.push_back(atoi(classLabel.c_str()));
      }
   }
}

cv::Rect detectFace(cv::Mat image)
{
   clock_t start;
   double duration;

   // Load face cascade 
   CascadeClassifier face_cascade;
   face_cascade.load("haarcascade_frontalface_alt.xml");

   // Detect face
   std::vector<Rect> face;
   cv::Rect roi;

   // measure detection time
   start = clock();

   face_cascade.detectMultiScale(image, face, HAAR_SCALE_FACTOR, HAAR_MIN_NEIGHBORS, 0|CV_HAAR_SCALE_IMAGE, Size(HAAR_MIN_SIZE, HAAR_MIN_SIZE));

   duration = (clock() - start) / (double) CLOCKS_PER_SEC;
   std::cout << "Face Detection Time: " << duration << std::endl;

   // did we succeed?
   if (face.size() != 1) {
      printf("No face detected\n");
      printf("Please try again\n");
      roi.x = 0;
      roi.y = 0;
      roi.height = 0;
      roi.width = 0;
      return roi;
   }
   // else {
   //    // print out rectangle
   //    std::cout << face[0] << std::endl;
   // }

   // return area of interest
   roi = face[0];
   return roi;
}

cv::Mat cropAndResizeImage(cv::Mat image, cv::Rect roi)
{
   cv::Mat ROI(image, roi);
   cv::Mat croppedImage;
   
   // set image ROI based on 
   croppedImage = image(Rect(roi.x, roi.y, roi.width, roi.height));

   // resize image to match training set
   cv::Mat finalImage;
   Size size(FACE_WIDTH, FACE_HEIGHT);
   cv::resize(croppedImage, finalImage, size);

   // return final cropped and resized image
   return finalImage;
}

Ptr<FaceRecognizer> trainModel(std::string fn_csv, std::string user_name) // vector<Mat> images, vector<int> labels)
{
   vector<Mat> images;
   vector<int> labels;
   clock_t start;
   double duration;

   // Read in the data from CSV file, which will have all users and negatives
   try {
      read_csv(fn_csv, images, labels);
   }
   catch (cv::Exception& e) {
      std::cerr << "Error opening file\"" << fn_csv << "\". Reason: " << e.msg << std::endl;
      exit(1);
   }

   // Need enough images for training
   if (images.size() <= 1) {
      std::cerr << "Need more than 1 images to train model" << std::endl;
   }
   
   // train model
   std::cout << "Training EigenFaceRecognizer Model" << std::endl;
   Ptr<FaceRecognizer> model = createEigenFaceRecognizer();
   // measure training time
   start = clock();
   model->train(images, labels);
   duration = (clock() - start) / (double) CLOCKS_PER_SEC;
   std::cout << "Training duration: " << duration << " seconds" << std::endl;

   // save model results
   model->save("training_" + user_name + ".xml");

   // return model object
   return model;
}
