// Program to detect a face using haar cascade classifier
// Andrew Palmer (ajp294)
// 21 October 2016
// Used tutorial on opencv for eigenfacerecognizer 

#include <iostream>
#include <fstream>
#include <sstream>
#include <sys/types.h>
#include <sys/stat.h>

#include "opencv2/objdetect/objdetect.hpp"
#include "opencv2/core/core.hpp"
#include "opencv2/highgui/highgui.hpp"
#include "opencv2/imgproc/imgproc.hpp"
#include "opencv2/contrib/contrib.hpp" // FaceRecognizer

#include "config.h"

#include "capture.cpp"
#include "faceDetect.cpp"
#include "captureAndTrain.cpp"
#include "recognizeAndAccess.cpp"

using namespace cv;
using namespace std;

int main(int argc, char ** argv) 
{
   vector<std::string> user_names;
   vector<Ptr<FaceRecognizer> > user_models;
   char user_input;

   cout << "Main Program Execution" << endl;
   
   // load users from txt file (Users.txt)
   user_names = loadUsersFromFile();
   if (user_names.size() >= 1) {
      // Load user models into memory
      user_models = loadUserModels();
      
      std::cout << "Finished loading user models" << std::endl;
   }

   // Main execution loop
   while(1) {
      std::cout << "To add a new user, enter a.  To gain access to building, enter e" << std::endl;
      std::cout << "Enter q to quit program" << std::endl;
      std::cin >> user_input;

      std::string new_user;
      Ptr<FaceRecognizer> new_user_model;
      cv::Mat testImage;

      switch(user_input) 
      {
         case('a'): 
            // captureAndTrain();
            new_user = newUser();   
            std::cout << "Finished adding new user" << std::endl;
            // repopulate users list and load new user model
            // user_names = loadUsersFromFile();
            user_names.push_back(new_user);
            
            new_user_model = trainModel(new_user);
            // user_models = loadUserModels();
            user_models.push_back(new_user_model);
            break;

         case('e'):
            // attempt to gain access to building
            testImage = captureDetectProcess();
            if (testImage.size().height == 0) {
               break;
            }
            recognizeAndAccess(testImage, user_models, user_names);
 
            break;
            
         case('q'):
            exit(1);
      }

   }
   
}
