// Module with functions for recognition and gaining door access
// Andrew Palmer (ajp294)
// 31 October 2016

#include <iostream>
#include <fstream>

// #include "faceDetect.cpp"

cv::Mat captureDetectProcess(void)
{
   cv::Mat testImage, processedTestImage;
   cv::Rect roi;

   // capture test image
   testImage = captureImage();

   // Detect face and crop/resize
   roi = detectFace(testImage);
   if (roi.width == 0) {
      return processedTestImage; // empty matrix returned because failed detection
   }
   processedTestImage = cropAndResizeImage(testImage, roi);

   return processedTestImage;
   
}

std::vector<std::string> loadUsersFromFile(void)
{
   // vector to store user names
   vector<std::string> user_name_list;

   // open users file
   std::ifstream infile;
   if (!(infile.open("Users.txt"), std::ios::in)) {
      std::cerr << "Cannot open input file" << std::endl;
   }
 
   // each entry in the users file is just a string so can parse easily knowing that
   std::string user_name;
   while (infile >> user_name) {
      user_name_list.push_back(user_name);
   }

   // return user names list
   return user_name_list;
}

vector<Ptr<FaceRecognizer> > loadUserModels(void)
{
   clock_t start;
   double duration;

   std::cout << "Loading models from user database" << std::endl;

   // vector of user models to load
   vector<Ptr<FaceRecognizer> > user_models;
   Ptr<FaceRecognizer> model = createEigenFaceRecognizer();

   // vector of user names
   std::vector<std::string> user_names = loadUsersFromFile();
   for (int i = 0 ; i < user_names.size(); i ++ ) {
      std::cout << "loading: training_" << user_names[i] << ".xml" << std::endl;

      start = clock();
      model->load("training_" + user_names[i] + ".xml");
      duration = (clock() - start ) / (double) CLOCKS_PER_SEC;
      std::cout << "Loading model duration: " << duration << " seconds" << std::endl;

      user_models.push_back(model);
      std::cout << "Finished loading " << user_names[i] << "'s facial recognition model" << std::endl;
   }

   return user_models;
}

bool recognizeAndAccess(cv::Mat testImage, vector<Ptr<FaceRecognizer> > user_models, vector<std::string> user_names)
{
   bool user_found = false;
   int predictedLabel = 1; // FINDME: starting at one because that's what user models are indexed at
   double confidence = 0.0;
   clock_t start;
   double duration;

   if (user_models.size() == 0) {
      std::cout << "No users in database. Please add new user" << std::endl;
   }


   // Check for a match
   std::cout << "Checking for match..." << std::endl;
   Ptr<FaceRecognizer> current_model;
   for (int i = 0 ; i < user_models.size() ; i ++ ) {
      // predictedLabel = user_models[i]->predict(testImage); // , i+1, confidence);
      //   i might have to be user_models.size() - 1 because of how labels are matched to models
      // std::cout << user_models.size() << " user models " << user_names.size() << " user names" << std::endl;
      // std::cout << "test image size: " << testImage.size() << std::endl;
      current_model = user_models[i];
      // user_models[i]->predict(testImage, predictedLabel, confidence); // , i+1, confidence);
      start = clock();
      current_model->predict(testImage, predictedLabel, confidence); // , i+1, confidence);
      duration = (clock() - start) / (double) CLOCKS_PER_SEC;
      std::cout << "Prediction duration: " << duration << " seconds" << std::endl;
      
      // predictedLabel = current_model->predict(testImage);
      std::cout << "Predicted label: " << predictedLabel << " with confidence: " << confidence << std::endl;
      if (confidence <= POSITIVE_THRESHOLD) {
         // std::cout << "Recognized face! Hello " << user_names[i] << std::endl;
         std::cout << "Recognized face! Hello " << user_names[predictedLabel - 1] << std::endl;
         std::cout << "Access granted" << std::endl;
          
         // when found a match, testing complete
         user_found = true;
         break;
      }
   }

   if (user_found) {
      std::cout << "Sending wireless access message" << std::endl;
      system("php /home/pi/meng/remote.php");
   }
   else {
      std::cout << "User not found.  Access denied" << std::endl;
   }
    
   return user_found;   
}

