"""Raspberry Pi Face Recognition Coffee Maker
Program to recognize a user and dispense coffee products based on the 
user's predefined preferences

Copyright 2016 Andrew Palmer (ajp294) 

"""
import cv2

import config
import face
import user
import sys
import RPi.GPIO as GPIO
import time
import subprocess

# Initialize camera
camera = config.get_camera()
print 'Press Ctrl-C to quit.'

def captureAndCrop():

   result = None
   # Block execution until we receive a usable image
   while(result is None):
      print("Looking for face...")
      
      # Capture images for comparison to models
      image = camera.read()
      # Convert image to greyscale
      image = cv2.cvtColor(image, cv2.COLOR_RGB2GRAY)
      # Get coordinates of single face in capture frame
      result = face.detect_single(image)
      # print(result)
      if result is None:
         print("Could not detect valid face!")
   
   x, y, w, h = result

   # Crop and resize valid image
   crop = face.resize(face.crop(image, x, y, w, h))
   return crop

def loadUserModels():

   # Load training data into model
   print("Loading models from user database...")
   user_preferences = [] # An array of arrays: [['andrew', 0, 1], ['joe', 1, 2]]
   user_models = [] # Store array of models 
   with open(config.USERS_FILE, 'r') as file:
      index = 0 # not very "pythonic" but whatever
      for line in file:
         user_data = line.split()
         user_preferences.append(user_data) 

         # Do the model instantiation and model.load in here for each user
         # FINDME: try the Pool library for parallelization
         model = cv2.createEigenFaceRecognizer()
         # Time each model's load duration
         # start = time.time()
         start = time.clock()
         model.load(config.TRAINING_FILE + '_' + user_preferences[index][0] + '.xml')
         duration = time.clock() - start
         print("Model load duration: " + str(duration) + " seconds")         

         user_models.append(model)
         print(user_preferences[index][0] + "'s model loaded")
         index += 1

   return user_preferences, user_models
   
def recognizeAndAccess(user_preferences, user_models):

   user_found = False

   if len(user_models) == 0:
      print("No users in database.  Please add new user")

   # Check model list twice
   for check_count in range(2):
      
      print("Checking for positive match in user models...")

      test_image = captureAndCrop()

      for model_index in range(len(user_models)):
         # Time how long a prediction takes
         # start = time.time()
         start = time.clock()
         label, confidence = user_models[model_index].predict(test_image)
         duration = time.clock() - start
         print("Prediction duration: " + str(duration) + " seconds")
      
         print("Checking image against model: " + user_preferences[model_index][0])

         print 'Predicted {0} face with confidence {1} (lower is more confident).'.format(
                'POSITIVE' if label == config.POSITIVE_LABEL else 'NEGATIVE', 
                confidence)

         # Look for face match
         if label == config.POSITIVE_LABEL and confidence < config.POSITIVE_THRESHOLD:
            print("Recognized face! Hello, " + user_preferences[model_index][0])

            # FINDME: removing these two lines so i can test confidence without breaking out of loop
            user_found = True
            break # When found a match, testing complete
         else:
            print('Did not recognize face! Checking next model in user list')
            # print("Attempt " + str(check_count + 1) + "/5")

      if (user_found):
         # Allow access into Home
         # Send RF message to Arduino to allow access
         # FINDME: When converting to C++, can just call my RF application normally
         subprocess.call("php /home/pi/meng/remote.php", shell=True)
         break


def main():

   user_preferences, user_models = loadUserModels()

   recognizeAndAccess(user_preferences, user_models)

if __name__ == '__main__': main()
